@@ -1,3 +1,4 @@
+import datetime
 import sys
 import logging
 import json
@@ -9,18 +10,13 @@ import socket
 import math
 from logging.handlers import DatagramHandler
 
-
+PY3 = sys.version_info[0] == 3
 WAN_CHUNK, LAN_CHUNK = 1420, 8154
 
-PY3 = sys.version_info[0] == 3
-
-
 if PY3:
-    string_type = str
-    integer_type = int,
+    data, text = bytes, str
 else:
-    string_type = basestring
-    integer_type = (int, long)
+    data, text = str, unicode
 
 
 class GELFHandler(DatagramHandler):
@@ -34,22 +30,27 @@ class GELFHandler(DatagramHandler):
     :param debugging_fields: Send debug fields if true (the default).
     :param extra_fields: Send extra fields on the log record to graylog
         if true (the default).
-    :param fqdn: Use fully qualified domain name of localhost as source 
+    :param fqdn: Use fully qualified domain name of localhost as source
         host (socket.getfqdn()).
     :param localname: Use specified hostname as source host.
     :param facility: Replace facility with specified value. If specified,
         record.name will be passed as `logger` parameter.
+    :param level_names: Allows the use of string error level names instead
+        of numerical values. Defaults to False
+    :param compress: Use message compression. Defaults to True
     """
 
     def __init__(self, host, port=12201, chunk_size=WAN_CHUNK,
-            debugging_fields=True, extra_fields=True, fqdn=False, 
-            localname=None, facility=None):
+            debugging_fields=True, extra_fields=True, fqdn=False,
+            localname=None, facility=None, level_names=False, compress=True):
         self.debugging_fields = debugging_fields
         self.extra_fields = extra_fields
         self.chunk_size = chunk_size
         self.fqdn = fqdn
         self.localname = localname
         self.facility = facility
+        self.level_names = level_names
+        self.compress = compress
         DatagramHandler.__init__(self, host, port)
 
     def send(self, s):
@@ -61,9 +62,11 @@ class GELFHandler(DatagramHandler):
 
     def makePickle(self, record):
         message_dict = make_message_dict(
-            record, self.debugging_fields, self.extra_fields, self.fqdn, 
-	    self.localname, self.facility)
-        return zlib.compress(json.dumps(message_dict).encode('utf-8'))
+            record, self.debugging_fields, self.extra_fields, self.fqdn,
+            self.localname, self.level_names, self.facility)
+        packed = message_to_pickle(message_dict)
+        frame = zlib.compress(packed) if self.compress else packed
+        return frame
 
 
 class ChunkedGELF(object):
@@ -91,7 +94,8 @@ class ChunkedGELF(object):
             yield self.encode(sequence, chunk)
 
 
-def make_message_dict(record, debugging_fields, extra_fields, fqdn, localname, facility=None):
+def make_message_dict(record, debugging_fields, extra_fields, fqdn, localname,
+                      level_names, facility=None):
     if fqdn:
         host = socket.getfqdn()
     elif localname:
@@ -107,6 +111,9 @@ def make_message_dict(record, debugging_fields, extra_fields, fqdn, localname, f
         'facility': facility or record.name,
     }
 
+    if level_names:
+        fields['level_name'] = logging.getLevelName(record.levelno)
+
     if facility is not None:
         fields.update({
             '_logger': record.name
@@ -155,9 +162,30 @@ def add_extra_fields(message_dict, record):
 
     for key, value in record.__dict__.items():
         if key not in skip_list and not key.startswith('_'):
-            if isinstance(value, (string_type, float) + integer_type):
-                message_dict['_%s' % key] = value
-            else:
-                message_dict['_%s' % key] = repr(value)
-
+            message_dict['_%s' % key] = value
     return message_dict
+
+
+def smarter_repr(obj):
+    """ convert JSON incompatible object to string"""
+    if isinstance(obj, datetime.datetime):
+        return obj.isoformat()
+    return repr(obj)
+
+
+def message_to_pickle(obj):
+    """ convert object to a JSON-encoded string"""
+    obj = sanitize(obj)
+    serialized = json.dumps(obj, separators=',:', default=smarter_repr)
+    return serialized.encode('utf-8')
+
+
+def sanitize(obj):
+    """ convert all strings records of the object to unicode """
+    if isinstance(obj, dict):
+        return dict((sanitize(k), sanitize(v)) for k, v in obj.items())
+    if isinstance(obj, (list, tuple)):
+        return obj.__class__([sanitize(i) for i in obj])
+    if isinstance(obj, data):
+        obj = obj.decode('utf-8', errors='replace')
+    return obj
