@@ -3,8 +3,12 @@ from amqplib import client_0_8 as amqp
 from graypy.handler import make_message_dict
 from logging import Filter
 from logging.handlers import SocketHandler
-from urlparse import urlparse
-import urllib
+
+try:
+    from urllib.parse import urlparse, unquote
+except ImportError:
+    from urlparse import urlparse
+    from urllib import unquote
 
 
 _ifnone = lambda v, x: x if v is None else v
@@ -39,7 +43,7 @@ class GELFRabbitHandler(SocketHandler):
             raise ValueError('invalid URL scheme (expected "amqp"): %s' % url)
         host = parsed.hostname or 'localhost'
         port = _ifnone(parsed.port, 5672)
-        virtual_host = virtual_host if not urllib.unquote(parsed.path[1:]) else urllib.unquote(parsed.path[1:])
+        virtual_host = virtual_host if not unquote(parsed.path[1:]) else unquote(parsed.path[1:])
         self.cn_args = {
             'host': '%s:%s' % (host, port),
             'userid': _ifnone(parsed.username, 'guest'),
